@@ -8,8 +8,7 @@ use 5.10.0;
 my $bot = WWW::Crawler::Mojo->new;
 $bot->on(res => sub {
     my ($bot, $scrape, $job, $res) = @_;
-    say sprintf('fetching %s resulted status %s',
-                                    $job->resolved_uri, $res->code);
+    say sprintf('fetching %s resulted status %s', $job->url, $res->code);
     $scrape->();
 });
 $bot->on(refer => sub {
